/* Function arrange_fit */

#include "attach_std_arf.i"
#include "prep_arf.i"
#include "rebin_specrmf.i"

func arrange_fit( specfile, effafile, rmffile=, rebin=, offaxis=, det=, id=, aps=, effabundle=, nofit= )
/* DOCUMENT arrange_fit, specfile[, effafile], rmffile=, rebin=, offaxis=, det=, id=, aps=, effabundle=, nofit=

    produces a stdfit_<id>.xcm file ready for XSPEC and runs it.

    Options: 1) A fit is to be done with user's choice of effective area
             2) A fit is to be done with ANCRFILE already in spectral file
             3) A fit is to be done with standard off axis effective area
                              by default or as given in 'effabundle'

    Arguments:
       specfile    the spectral file in PHAII format
       effafile    1) the effective area file (with columns ENERG_LO,
                      ENERG_HI, EFF_AREA).
                   2) omit this argument as well as keyword 'offaxis'
                   3) omit this argument and set the keyword 'offaxis'
    Keywords:
       rebin       The rebinning array to use. If given a new
                   spectral file with a rebinned spectrum is produced.
                   The RMF file is also produced in a rebinned version.
       offaxis     [arcmin] only to be given if 'effafile' is not given
       det         Number of detector (0 .. 3)
       id          Identification for fit, cannot contain spaces
       aps         Aperture stop correction file for option 1) and 3)
       nofit       Inhibit launching the call of xspec.
       effabundle  The assembled effective area file such as:
                     /home/njw/0idl/imd/pf16_sat00_105_aps3/Eff_area_FM2_v12_aps3_130308.fits
                     which is the default filename.
*/
{
   if( !file_test(specfile) ) error,"Spectral file was not found";

   // Get the instrument

   hdr = headfits(specfile+"+1");
   instrume = fxpar( hdr, "instrume" );
   if( instrume == "FPMA" ) {
      fmodel = 2;
   } else if( instrume == "FPMB" ) {
      fmodel = 1;
   } else error,"Unknown INSTRUME in "+specfile;

   // See, if 'effafile' has been given as an argument

   option = 1;
   if( is_void(effafile) ) {
      option = is_void(offaxis) ? 2 : 3;
   } else {
      if( !file_test(effafile) ) error,"Eff. Area file was not found";
   }
   write,"ARRANGE_FIT: option = "+itoa(option);

   // Do the rebinning if required

   if( !is_void(rebin) ) {
      write,"ARRANGE_FIT: rebinning ...";
      rebin_specrmf, specfile, rebin=rebin;
      p = strpos( specfile, ".", rev=1 );
      specfile = strinsert( specfile, "_rebin", p); // same renaming as in 'rebin_specrmf'
   }

   if( option == 1 ) { // apply user's choice of effective area
      write,"ARRANGE_FIT: effafile = "+effafile;
      fullarfname = get_next_filename(instrume+"_????.arf");
      write,"ARRANGE_FIT: fullarfname = "+fullarfname;
      if( !is_void(aps) ) write,"ARRANGE_FIT: aps = "+aps;
      prep_arf, fmodel, effafile, fullarfname,aps=aps;
      write,"Has produced "+fullarfname;
      attach_arf, specfile, fullarfname;
  //  -- option 2 assumes that the ARF already attached is adequate
   } else if( option == 3 ) { // standard offaxis effective area
      attach_std_arf, specfile, offaxis, det=det, aps=aps, effabundle=effabundle;
   }

   // Read the fit template, adjust names and write into: stdfit_id.xcm

   if( is_void(id) ) {
      id = get_next_filename("fit????", next);
      system,"touch "+fit+next;
   }
   id = strstrrepl( id, " ", "_"); // avoid space since 'id' is used for file naming
   tpl = rdfile("stdfit_tpl.xcm");
   tpl = strstrrepl( tpl, "idenstring", id );
   tpl = strstrrepl( tpl, "specfile", specfile );
   tpl = strstrrepl( tpl, "fpmstring", instrume );

   write_slist,"stdfit_"+id+".xcm", tpl;

   write,"Now you can issue: xspec - stdfit_"+id;

   if( !nofit ) system,"xspec - stdfit_"+id;

}
