#include "read_energ_data.i"
#include "attach_std_arf.i"

func prep_spec( specfile, offaxis, rmffile=, det=, aps=, effabundle= )
/* DOCUMENT prep_spec, specfile, offaxis, rmffile=, det=, aps=, effabundle=

     Upgrade a spectral file in PHAI format to PHAII and supplement with an ARF properly
     corrected for off-axis angle, thermal foil absorption and aperture stop correction.
     Likewise an RMF file is attached.

     Assume that the spectral file 'specfile' is in PHAI format with only a 'counts' column.
     and a keyword 'EXPOSURE'.

     If no 'offaxis' value has been given then no ARF is attached (to be done in another process).
     The off-axis angle 'offaxis' is assumed to be in arcmin.

     The default telescope effective area file is:
           ~/0idl/imd/pf16_sat00_105_aps3/Eff_area_FM[12]_v12_aps3_130308.fits

     Keyword:    rmffile     Name/path of RMF file for RESPFILE keyword
                 det         the detector number (defaults to 0). Used for the standard
                             naming: nu[AB]cutdet[0123]_20100101v001.rmf
                             of the RMF file.
                 aps         The path/name of the aperture stop correction file.
                 effabundle  The assembled effective area file.

             NB: Giving 'rmffile' will override the 'det' indication

   2012-12-19/NJW, updated 2013-03-11
*/
{

  /*
   *    Convert to PHAII format
   */

   if( !file_test(specfile) ) error,"Did not find "+specfile;

   counts = rdfitscol( specfile+"+1","counts");
   hdr = headfits( specfile+"+1");
   exposure = fxpar( hdr, "exposure");
   instrume = fxpar( hdr, "instrume")(1);
   p = strpos( specfile, ".", rev=1 ); // locate last dot
   specfile_root = strpart( specfile, 1:p-1 );
   outfile = strinsert( specfile, "_phaii", p );
   rate = counts / exposure;
   stat_err = sqrt(max(counts,array(1.,numberof(counts)))) / exposure;
   ra_obj = 83.633;
   dec_obj = 22.015;
   name = "Crab";
   telescop = "NuSTAR";
   if( instrume == "FPMA" ) {
      letter = "A";
      mstr = "FM2";
   } else if( instrume == "FPMB" ) {
      letter = "B";
      mstr = "FM1";
   } else error,"Unknown instrument: "+instrume;
   arfname = specfile_root + "_phaii.arf";

   if( is_void(rmffile) ) {
      if( is_void(det) ) det = 0;
      respfile = "nu"+letter+"cutdet"+itoa(det)+"_20100101v001.rmf";
   } else respfile = rmffile;

   spec2phaii, outfile, rate, stat_err, \
      type="net", name=name, ra_obj=ra_obj, dec_obj=dec_obj, \
      exposure=exposure, \
      ancrfile=arfname, \
      respfile=respfile, \
      telescop=telescop, instrume=instrume;
   write,"Wrote "+outfile;

   if( is_void(offaxis) ) {
      write,"No further action since 'offaxis' was not given";
      return;
   }

  /*
   *    Construct the ARF from the optic effective area, the
   *    thermal foil absorption, and the aperture stop correction
   *    and attach it to the spectral file.
   */

   attach_std_arf, outfile, offaxis, det=det, aps=aps, effabundle=effabundle;
}
