/*******************************************************
  Rebin both a spectrum in PHAII format and the RDM in
  an RMF file

  2012-12-20/NJW
*********************************************************/

func rebin_specrmf( specfile, rmffile=, rebin=, frac=, outspecfile=, outrmffile= )
/* DOCUMENT rebin_specrmf, specfile, rmffile=, rebin=, frac=, outspecfile=, outrmffile=
           rebin = frac=, outspecfile=, outrmffile=

     The spectrum is assumed to be found in a PHAII format file.

     Default is to use 'RESPFILE' in specfile as the RMF, this is overridden by
          giving 'rmffile'

     'rebin' overrides 'frac'

     NB! The present version only works on the first spectrum in the file.
*/
{
   local ob1, ob2, orate, orate_err;

   if( !file_test(specfile) ) error,"File not found";

   rate = rdfitscol(specfile+"+1","rate")(,1);
   stat_err = rdfitscol(specfile+"+1","stat_err")(,1);
   ancrfile = rdfitscol(specfile+"+1","ancrfile")(1);
   name = rdfitscol(specfile+"+1","rowid")(1);
   ra_obj = rdfitscol(specfile+"+1","ra_obj")(1);
   dec_obj = rdfitscol(specfile+"+1","dec_obj")(1);
   exposure = rdfitscol( specfile+"+1", "exposure")(1);
   hdr = headfits(specfile+"+1");
   instrume = fxpar( hdr, "instrume")(1);
   telescop = fxpar( hdr, "telescop");

   if( is_void(rmffile) ) rmffile = fxpar( hdr, "RESPFILE" );

   dol = rmffile+"[MATRIX]";
   rdm = rdfitscol(dol,"MATRIX");
   elo = rdfitscol(dol,"ENERG_LO");
   ehi = rdfitscol(dol,"ENERG_HI");
   n_grp = rdfitscol(dol,"N_GRP");
   f_chan = rdfitscol(dol,"F_CHAN");
   n_chan = rdfitscol(dol,"N_CHAN");
   dol = rmffile+"[EBOUNDS]";
   e_min = rdfitscol(dol,"E_MIN");
   e_max = rdfitscol(dol,"E_MAX");

   if( is_void(rebin) ) { // Then create the 'rebin' array
      if( is_void(frac) ) frac = 0.2; // some reasonable value
      rebin = specrebinninga( e_min, e_max, rate, stat_err, frac, ob1, ob2, orate, orate_err );
   } else {   // apply the 'rebin' array directly
      write,"Applying the rebin array ...";
      specrebinning, e_min, e_max, rate, stat_err, rebin, ob1, ob2, orate, orate_err
   }

   // test for negative values except at the end of 'rebin'
   w = where(rebin <= 0);
   nw = numberof(w);
   if( nw ) {
      if( nw == 1 ) if( w(1) != numberof(rebin) ) error,"##1## dodgy rebin array";
      if( w(1) < numberof(rebin)/2 ) error,"##2## dodgy rebin array";
   }

   // Write the spectral file with rebinned spectra

   if( is_void(outspecfile) ) {
      p = strpos( specfile, ".", rev=1 );
      outspecfile = strinsert( specfile, "_rebin", p );
   }
   write,"outspecfile = "+outspecfile;
   if( is_void(outrmffile) ) {
      p = strpos( rmffile, ".", rev=1 );
      outrmffile = strinsert( rmffile, "_rebin", p );
   }
   fits_copy_keys, hdr, tokwds=1;

   spec2phaii, outspecfile, orate, orate_err, \
      type="net", name=name, ra_obj=ra_obj, dec_obj=dec_obj, \
      exposure=exposure, \
      ancrfile=ancrfile, \
      respfile=outrmffile, \
      telescop=telescop, instrume=instrume;
   write,"Wrote "+outspecfile;

   // Check if the suggested RMF file with the rebinned RDM already
   // exists and with the same rebinning

   write,"outrmffile = "+outrmffile;

   rewrite = 1;
   if( file_test( outrmffile) ) {
      write,"Found "+outrmffile;
      exist_rebin = rdfitscol( outrmffile+"[REBIN]", "rebin" );
      write,"numberof(rebin) = "+itoa(numberof(rebin));
      write,"numberof(exist_rebin) = "+itoa(numberof(exist_rebin));
      if( numberof(rebin) == numberof(exist_rebin) ) {
         write,"max(abs(rebin-exist_rebin)) = "+itoa(max(abs(rebin-exist_rebin)));
         if( noneof(rebin - exist_rebin) ) rewrite = 0;
      }
   }
   write,"'rewrite' = "+itoa(rewrite);

   if( rewrite ) {

      // Write the RMF file with rebinned RDM
      write,"Going to write "+outrmffile;

      rebinx = rebin;
      newrdm = rebin_rdm( rdm, rebinx );
      dms = dimsof(newrdm);

      dol = rmffile+"[MATRIX]";
      hdr = headfits(dol);
      fits_copy_keys,hdr,tokwds=1;
      n_chan() = dms(2);
      kwds_set,"DETCHANS", dms(2),"Number of detector channels";

      fh = wrmfitscols( outrmffile, "ENERG_LO",elo,"ENERG_HI",ehi,"N_GRP",n_grp, \
              "F_CHAN", f_chan, "N_CHAN", n_chan, "MATRIX", newrdm, cont=1, clobber=1 );

      // Write the EBOUNDS extension

      dol = rmffile+"[EBOUNDS]";
      hdr = headfits(dol);
      fits_copy_keys,hdr,tokwds=1;
      kwds_set,"DETCHANS", dms(2),"Number of detector channels";
      fh = wrmfitscols( fh, "E_MIN", ob1, "E_MAX", ob2, cont=1 );

      // Write extra REBIN extension with rebinning array

      kwds_set,"EXTNAME","REBIN","Name of this extension";
      wrmfitscols, fh, "REBIN", rebin;
      write,"Wrote "+outrmffile;

   } else write,"No rewriting required ...";

   return rebin;

}
